/*
 * Data space allocation utility functions.
 *
 * Copyright (C) 2018, 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/c/mem_alloc.h>
#include <l4/re/c/rm.h>

#include <ipc/mem_ipc.h>

#include "dataspace.h"



/* Allocate and map a region containing data, using the given mem_size to
   indicate the required region size, along with a pointer to be set to the
   allocated region. Return a capability which may be invalid if allocation or
   mapping failed.
*/

l4re_ds_t allocate_data(l4_size_t mem_size, void **allocated)
{
  l4re_ds_t mem;

  /* Allocate memory for the data and map it to a virtual address. */

  if (ipc_new_dataspace(mem_size, L4RE_MA_CONTINUOUS, 0, &mem))
    return L4_INVALID_CAP;

  if (ipc_attach_dataspace_align(mem, mem_size,
                                 L4RE_RM_F_SEARCH_ADDR | L4RE_RM_F_RW,
                                 0, allocated))
    return L4_INVALID_CAP;

  return mem;
}
